/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.uiextensions;

import android.graphics.PointF;
import android.graphics.RectF;
import android.view.KeyEvent;
import android.view.MotionEvent;

import com.foxit.sdk.ActionCallback;
import com.foxit.sdk.PDFException;
import com.foxit.sdk.PDFViewCtrl;
import com.foxit.sdk.common.Library;
import com.foxit.sdk.pdf.PDFDoc;
import com.foxit.sdk.pdf.PDFPage;
import com.foxit.sdk.pdf.Signature;
import com.foxit.sdk.pdf.annots.Annot;
import com.foxit.sdk.pdf.annots.Ink;
import com.foxit.sdk.pdf.annots.Markup;
import com.foxit.sdk.pdf.objects.PDFArray;
import com.foxit.sdk.pdf.objects.PDFDictionary;
import com.foxit.sdk.pdf.objects.PDFObject;
import com.foxit.uiextensions.annots.AnnotContent;
import com.foxit.uiextensions.annots.AnnotEventListener;
import com.foxit.uiextensions.annots.AnnotHandler;
import com.foxit.uiextensions.annots.IFlattenEventListener;
import com.foxit.uiextensions.config.permissions.PermissionsConfig;
import com.foxit.uiextensions.modules.textselect.BlankSelectToolHandler;
import com.foxit.uiextensions.modules.textselect.TextSelectToolHandler;
import com.foxit.uiextensions.utils.AppAnnotUtil;
import com.foxit.uiextensions.utils.AppDmUtil;
import com.foxit.uiextensions.utils.AppUtil;
import com.foxit.uiextensions.utils.Event;

import org.json.JSONArray;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import static com.foxit.uiextensions.utils.AppAnnotUtil.ANNOT_SELECT_TOLERANCE;
/**
 * Management for PDF document annotations, pages, permissions, and so on. This implementation is shared with all the modules.
 **/
public class DocumentManager extends AbstractUndo {
    private static final Lock lock = new ReentrantLock();

    protected Annot mCurAnnot = null;
//    private PDFViewCtrl mPdfViewCtrl;
    private WeakReference<PDFViewCtrl> mPdfViewCtrl;
    private ArrayList<AnnotEventListener> mAnnotEventListenerList = new ArrayList<AnnotEventListener>();
    private ArrayList<IFlattenEventListener> mFlattenEventListenerList = new ArrayList<IFlattenEventListener>();
    private ActionCallback mActionCallback = null;

    private Boolean isSetActionCallback = Boolean.valueOf(false);
    private long mUserPermission = 0;
    private int mMDPPermission = 0; // signature permission
    private boolean mIsSign = false;
    private boolean mHasModifyTask = false;
    private boolean mIsCDRM = false;

    private boolean mIsDocModified = false;

    protected DocumentManager on(PDFViewCtrl ctrl){
        AppUtil.requireNonNull(ctrl);
        if (getPdfViewCtrl() == null){
            mPdfViewCtrl = new WeakReference(ctrl);
        }
        return this;
    }

    private PDFViewCtrl getPdfViewCtrl() {
        return mPdfViewCtrl.get();
    }

    public DocumentManager(PDFViewCtrl pdfViewCtrl) {
        mPdfViewCtrl = new WeakReference<PDFViewCtrl>(pdfViewCtrl);
    }

    public void setActionCallback(ActionCallback handler) {
        lock.lock();
        if (isSetActionCallback) return;
        mActionCallback = handler;
        isSetActionCallback = Library.setActionCallback(mActionCallback);
        lock.unlock();
    }

    public void reInit() {
        lock.lock();
        isSetActionCallback = false;
        lock.unlock();
    }

    public ActionCallback getActionCallback() {
        return mActionCallback;
    }

    public void resetActionCallback() {
        lock.lock();
        isSetActionCallback = Library.setActionCallback(mActionCallback);
        lock.unlock();
    }

    public void setCurrentAnnot(Annot annot) {
        if (mCurAnnot == annot) return;
        Annot lastAnnot = mCurAnnot;
        PDFViewCtrl viewCtrl = getPdfViewCtrl();
        if (viewCtrl == null) return;
        UIExtensionsManager uiExtensionsManager = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager());
        if (annot == null) {
            uiExtensionsManager.startHideToolbarsTimer();
        } else {
            uiExtensionsManager.stopHideToolbarsTimer();
        }
        try {
            lock.lock();
            if (mCurAnnot != null && !mCurAnnot.isEmpty()) {

                int type = AppAnnotUtil.getAnnotHandlerType(lastAnnot);
                if (uiExtensionsManager.getAnnotHandlerByType(type) != null) {
                    uiExtensionsManager.getAnnotHandlerByType(type).onAnnotDeselected(lastAnnot, true);
                }
            }

            mCurAnnot = annot;
            if (annot != null && uiExtensionsManager.getAnnotHandlerByType(AppAnnotUtil.getAnnotHandlerType(annot)) != null) {
                if (annot.getUniqueID() == null) {
                    annot.setUniqueID(AppDmUtil.randomUUID(null));
                }
                uiExtensionsManager.getAnnotHandlerByType(AppAnnotUtil.getAnnotHandlerType(annot)).onAnnotSelected(annot, true);
            }
        } catch (PDFException e) {
            e.printStackTrace();
        } finally {
            lock.unlock();
        }
        onAnnotChanged(lastAnnot, mCurAnnot);
    }

    public Annot getCurrentAnnot() {
        return mCurAnnot;
    }

    protected Annot getFocusAnnot() {
        if (mCurAnnot != null && !mCurAnnot.isEmpty()) {
            return mCurAnnot;
        }

        if (mEraseAnnotList.size() > 0) {
            return mEraseAnnotList.get(0);
        }

        return null;
    }

    public boolean shouldViewCtrlDraw(Annot annot) {
        try {
            lock.lock();
            if (mCurAnnot != null && !mCurAnnot.isEmpty() && mCurAnnot.getPage().getIndex() == annot.getPage().getIndex()) {
                int type = AppAnnotUtil.getAnnotHandlerType(mCurAnnot);
                AnnotHandler annotHandler = ((UIExtensionsManager) getPdfViewCtrl().getUIExtensionsManager()).getAnnotHandlerByType(type);
                if (annotHandler != null) {
                    return annotHandler.shouldViewCtrlDraw(annot);
                }
            }

            //for eraser
            if (annot.getType() != Annot.e_Ink) return true;
            for (int i = 0; i < mEraseAnnotList.size(); i ++) {
                Ink ink =  mEraseAnnotList.get(i);
                if (ink.getPage().getIndex() == annot.getPage().getIndex() &&
                        ink.getIndex() == annot.getIndex()) {
                    return false;
                }
            }
        } catch (PDFException e) {
            e.printStackTrace();
        } finally {
            lock.unlock();
        }
        return true;
    }

    private int getMDPDigitalSignPermissionInDocument(PDFDoc document) throws PDFException {
        PDFDictionary catalog = document.getCatalog();
        if(catalog == null) return 0;
        PDFObject object = catalog.getElement("Perms");
        if (object == null)  return 0;

        PDFDictionary perms = object.getDirectObject().getDict();
        if (perms == null) return 0;

        object = perms.getElement("DocMDP");
        if (object == null) return 0;

        PDFDictionary docMDP = object.getDirectObject().getDict();
        if (docMDP == null) return 0;

        object = docMDP.getElement("Reference");
        if (object == null) return 0;

        PDFArray reference = object.getDirectObject().getArray();
        if (reference == null) return 0;

        for (int i = 0; i < reference.getElementCount(); i++) {
            object = reference.getElement(i);
            if (object == null) return 0;

            PDFDictionary tmpDict = object.getDirectObject().getDict();
            if (tmpDict == null) return 0;

            object = tmpDict.getElement("TransformMethod");
            if (object == null) return 0;

            String transformMethod = object.getDirectObject().getWideString();
            if (!transformMethod.contentEquals("DocMDP")) continue;

            object = tmpDict.getElement("TransformParams");
            if (object == null) return 0;

            PDFDictionary transformParams = object.getDirectObject().getDict();
            if (transformParams == null || transformParams == tmpDict) return 0;

            object = transformParams.getElement("P");
            if (object == null) return 0;

            return object.getDirectObject().getInteger();
        }
        return 0;
    }

    private boolean isDocOpen() {
        if (getPdfViewCtrl() == null || getPdfViewCtrl().getDoc() == null) return false;
        return true;
    }

    public boolean canPrint() {
        if (!isDocOpen()) return false;
        if (getPdfViewCtrl().isOwner()) return true;
        if (mIsCDRM) {
            return (mCDRMPermission & PDFDoc.e_PermPrint) != 0;
        }

        if (getPdfViewCtrl().isDynamicXFA()) return true;
        return (mUserPermission & PDFDoc.e_PermPrint) != 0;
    }

    public boolean canPrintHighQuality() {
        if (!isDocOpen()) return false;
        if (getPdfViewCtrl().isOwner()) return true;
        if (mIsCDRM) {
            return (mCDRMPermission & PDFDoc.e_PermPrint) != 0 ||
                    (mCDRMPermission & PDFDoc.e_PermPrintHigh) != 0;
        }

        return (mUserPermission & PDFDoc.e_PermPrint) != 0 ||
                (mUserPermission & PDFDoc.e_PermPrintHigh) != 0;
    }

    public boolean canCopy() {
        UIExtensionsManager uiExtensionsManager = (UIExtensionsManager)getPdfViewCtrl().getUIExtensionsManager();
        PermissionsConfig permissionsConfig = uiExtensionsManager.getConfig().permissions;
        if (!permissionsConfig.copyText) return false;

        if (!isDocOpen()) return false;
        if (getPdfViewCtrl().isOwner()) return true;
        if (mIsCDRM) {
            return (mCDRMPermission & PDFDoc.e_PermExtract) != 0;
        }

        return (mUserPermission & PDFDoc.e_PermExtract) != 0;
    }

    public boolean canCopyForAssess() {
        if (!isDocOpen()) return false;
        if (getPdfViewCtrl().isOwner()) return true;
        if (mIsCDRM) {
            return (mCDRMPermission & PDFDoc.e_PermExtractAccess) != 0 || (mCDRMPermission & PDFDoc.e_PermExtract) != 0;
        }
        return (mUserPermission & PDFDoc.e_PermExtractAccess) != 0 || (mUserPermission & PDFDoc.e_PermExtract) != 0;
    }

    public boolean canAssemble() {
        if (!isDocOpen()) return false;
        if (mIsCDRM) {
            if (getPdfViewCtrl().isOwner()) return true;
            return (mCDRMPermission & PDFDoc.e_PermAssemble) != 0
                    || (mCDRMPermission & PDFDoc.e_PermModify) != 0;
        }

        if (!canModifyFile() && !canSaveAsFile())  return false;

        UIExtensionsManager uiExtensionsManager = (UIExtensionsManager)getPdfViewCtrl().getUIExtensionsManager();
        if(uiExtensionsManager != null && uiExtensionsManager.getModuleByName(Module.MODULE_NAME_DIGITALSIGNATURE) != null) {
            if (mMDPPermission != 0) return false;
            if (mIsSign) return false;
        }
        if (getPdfViewCtrl().isOwner()) return true;
        return (mUserPermission & PDFDoc.e_PermAssemble) != 0
                || (mUserPermission & PDFDoc.e_PermModify) != 0;
    }

    public boolean canModifyContents() {
        return canModifyContents(false);
    }

    private boolean canModifyContents(boolean isJudgeAddSignature) {
        if (!isDocOpen()) return false;

        if (mIsCDRM) {
            if (getPdfViewCtrl().isOwner()) return true;
            return (mCDRMPermission & PDFDoc.e_PermModify) != 0;
        }

        if (!canModifyFile() && !canSaveAsFile()) return false;

        UIExtensionsManager uiExtensionsManager = (UIExtensionsManager)getPdfViewCtrl().getUIExtensionsManager();
        if(uiExtensionsManager != null && uiExtensionsManager.getModuleByName(Module.MODULE_NAME_DIGITALSIGNATURE)!= null) {
            if (mMDPPermission != 0) return false;
            if (!isJudgeAddSignature && mIsSign) return false;
        }
        if (getPdfViewCtrl().isOwner()) return true;
        return (mUserPermission & PDFDoc.e_PermModify) != 0;
    }

    public boolean canFillForm() {
        if (!isDocOpen()) return false;
        if (mIsCDRM) {
            if (getPdfViewCtrl().isOwner()) return true;
            return (mCDRMPermission & PDFDoc.e_PermFillForm) != 0
                    || (mCDRMPermission & PDFDoc.e_PermAnnotForm) != 0
                    || (mCDRMPermission & PDFDoc.e_PermModify) != 0;
        }

        if (!canModifyFile() && !canSaveAsFile()) return false;

        UIExtensionsManager uiExtensionsManager = (UIExtensionsManager)getPdfViewCtrl().getUIExtensionsManager();
        if(uiExtensionsManager != null && uiExtensionsManager.getModuleByName(Module.MODULE_NAME_DIGITALSIGNATURE)!= null) {
            if (mMDPPermission == 1) return false;
        }
        if (getPdfViewCtrl().isOwner()) return true;
        return (mUserPermission & PDFDoc.e_PermFillForm) != 0
                || (mUserPermission & PDFDoc.e_PermAnnotForm) != 0
                || (mUserPermission & PDFDoc.e_PermModify) != 0;
    }

    public boolean canAddAnnot() {
        if (!isDocOpen()) return false;
        if (getPdfViewCtrl().isDynamicXFA()) return false;

        if (mIsCDRM) {
            if (getPdfViewCtrl().isOwner()) return true;
            return (mCDRMPermission & PDFDoc.e_PermAnnotForm) != 0;
        }

        if (!canModifyFile() && !canSaveAsFile())  return false;

        UIExtensionsManager uiExtensionsManager = (UIExtensionsManager)getPdfViewCtrl().getUIExtensionsManager();
        if(uiExtensionsManager != null && uiExtensionsManager.getModuleByName(Module.MODULE_NAME_DIGITALSIGNATURE)!= null) {
            if (mMDPPermission == 1 || mMDPPermission == 2) return false;
        }
        if (getPdfViewCtrl().isOwner()) return true;
        return (mUserPermission & PDFDoc.e_PermAnnotForm) != 0;
    }

    public boolean canSigning() {
            if (mMDPPermission > 0) return false;
//        if (isSign()) return false;
        if (canAddAnnot() || canFillForm()) return true;
        return false;
    }

    public boolean canAddSignature() {
        if (!isDocOpen()) return false;
        if (getPdfViewCtrl().isDynamicXFA()) return false;
        if (isSign() && !canAddAnnot()) return false;
        if (canSigning() && canModifyContents(true)) return true;
        return false;
    }

    public boolean canModifyFile() {
        if (!isDocOpen()) return false;
        return true;
    }

    public boolean canSaveAsFile() {
        if (!isDocOpen()) return false;
        return true;
    }

    public boolean isSign() {
        if (!isDocOpen()) return false;
        return mIsSign;
    }

    public boolean isXFA() {
        if (!isDocOpen()) return false;
        try {
            return getPdfViewCtrl().getDoc().isXFA();
        } catch (PDFException e) {
            e.printStackTrace();
        }
        return false;
    }

    // annot event listener management.
    public void registerAnnotEventListener(AnnotEventListener listener) {
        mAnnotEventListenerList.add(listener);
    }

    public void unregisterAnnotEventListener(AnnotEventListener listener) {
        mAnnotEventListenerList.remove(listener);
    }

    public void onAnnotAdded(PDFPage page, Annot annot) {
        for (AnnotEventListener listener : mAnnotEventListenerList) {
            listener.onAnnotAdded(page, annot);
        }
    }

    public void onAnnotWillDelete(PDFPage page, Annot annot) {
        for (AnnotEventListener listener : mAnnotEventListenerList) {
            listener.onAnnotWillDelete(page, annot);
        }
    }

    public void onAnnotDeleted(PDFPage page, Annot annot) {
        for (AnnotEventListener listener : mAnnotEventListenerList) {
            listener.onAnnotDeleted(page, annot);
        }
    }

    public void onAnnotModified(PDFPage page, Annot annot) {
        for (AnnotEventListener listener : mAnnotEventListenerList) {
            listener.onAnnotModified(page, annot);
        }
    }

    public void onAnnotChanged(Annot lastAnnot, Annot currentAnnot) {
        for (AnnotEventListener listener : mAnnotEventListenerList) {
            listener.onAnnotChanged(lastAnnot, currentAnnot);
        }
    }

    public void registerFlattenEventListener(IFlattenEventListener listener) {
        mFlattenEventListenerList.add(listener);
    }

    public void unregisterFlattenEventListener(IFlattenEventListener listener) {
        mFlattenEventListenerList.remove(listener);
    }

    public void onAnnotWillFlatten(PDFPage page, Annot annot) {
        for (IFlattenEventListener listener : mFlattenEventListenerList) {
            listener.onAnnotWillFlatten(page, annot);
        }
    }

    public void onAnnotFlattened(PDFPage page, Annot annot) {
        for (IFlattenEventListener listener : mFlattenEventListenerList) {
            listener.onAnnotFlattened(page, annot);
        }
    }

    protected List<Ink> mEraseAnnotList = new ArrayList<Ink>();
    public void onAnnotStartEraser(Ink annot) {
        mEraseAnnotList.add(annot);
    }

    public void onAnnotEndEraser(Ink annot) {
        mEraseAnnotList.remove(annot);
    }

    public static  boolean intersects(RectF a, RectF b) {
        return a.left < b.right && b.left < a.right
                && a.top > b.bottom && b.top > a.bottom;
    }

    public ArrayList<Annot> getAnnotsInteractRect(PDFPage page, RectF rect, int type) {
        ArrayList<Annot> annotList = new ArrayList<Annot>(4);
        try {
            int count = page.getAnnotCount();
            Annot annot = null;
            for (int i = 0; i < count; i++) {
                annot = AppAnnotUtil.createAnnot(page.getAnnot(i));
                if (annot == null || (annot.getFlags() & Annot.e_FlagHidden) != 0)
                    continue;

                PDFViewCtrl viewCtrl = getPdfViewCtrl();
                if (viewCtrl != null){
                    int _type = AppAnnotUtil.getAnnotHandlerType(annot);
                    AnnotHandler annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(_type);
                    if (annotHandler != null) {
                        RectF bbox = annotHandler.getAnnotBBox(annot);
                        if (intersects(bbox, rect) && annot.getType() == type) {
                            annotList.add(annot);
                        }
                    }
                }
            }
        } catch (PDFException e) {
            e.printStackTrace();
        }

        return annotList;
    }

    public Annot getAnnot(PDFPage page, String nm) {
        if (page == null || page.isEmpty()) return null;

        try {
            int count = page.getAnnotCount();
            Annot annot = null;
            for (int i = 0; i < count; i++) {
                annot = AppAnnotUtil.createAnnot(page.getAnnot(i));
                if (annot == null || annot.isEmpty() || annot.getUniqueID() == null) continue;

                if (annot.getUniqueID().equals(nm)) return annot;
            }
        } catch (PDFException e) {
            e.printStackTrace();
        }

        return null;
    }

    public void addAnnot(PDFPage page, AnnotContent content, boolean addUndo, Event.Callback result) {
        if (page == null || page.isEmpty()) return;

        Annot annot = getAnnot(page, content.getNM());
        if (annot != null && !annot.isEmpty()) {
            modifyAnnot(annot, content, addUndo, result);
            return;
        }
        PDFViewCtrl viewCtrl = getPdfViewCtrl();
        if (viewCtrl != null){
            try {

                UIExtensionsManager uiExtensionsManager = (UIExtensionsManager) viewCtrl.getUIExtensionsManager();
                AnnotHandler annotHandler = uiExtensionsManager.getAnnotHandlerByType(content.getType());
                if (annotHandler != null) {
                    annotHandler.addAnnot(page.getIndex(), content, addUndo, result);
                } else {
                    if (result != null) {
                        result.result(null, false);
                    }
                }
            }catch (PDFException e){
                e.printStackTrace();
            }
        }
    }

    public void modifyAnnot(Annot annot, AnnotContent content, boolean addUndo, Event.Callback result) {
        try {
            if (annot.getModifiedDateTime() != null && content.getModifiedDate() != null
                    && annot.getModifiedDateTime().equals(content.getModifiedDate())) {
                if (result != null) {
                    result.result(null, true);
                }
                return;
            }

            PDFViewCtrl viewCtrl = getPdfViewCtrl();
            if (viewCtrl != null){
                UIExtensionsManager uiExtensionsManager = (UIExtensionsManager) viewCtrl.getUIExtensionsManager();
                AnnotHandler annotHandler = uiExtensionsManager.getAnnotHandlerByType(AppAnnotUtil.getAnnotHandlerType(annot));
                if (annotHandler != null) {
                    annotHandler.modifyAnnot(annot, content, addUndo, result);
                } else {
                    if (result != null) {
                        result.result(null, false);
                    }
                }
            }
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    public void removeAnnot(final Annot annot, boolean addUndo, final Event.Callback result) {
        if (annot == getCurrentAnnot()) {
            setCurrentAnnot(null);
        }

        PDFViewCtrl viewCtrl = getPdfViewCtrl();
        if (viewCtrl != null){
            UIExtensionsManager uiExtensionsManager = (UIExtensionsManager) viewCtrl.getUIExtensionsManager();
            AnnotHandler annotHandler = uiExtensionsManager.getAnnotHandlerByType(AppAnnotUtil.getAnnotHandlerType(annot));

            if (annotHandler != null) {
                annotHandler.removeAnnot(annot, addUndo, result);
            }
        }
    }

    //deal with annot
    public boolean onTouchEvent(int pageIndex, MotionEvent motionEvent) {
        Annot annot = null;
        AnnotHandler annotHandler = null;
        PDFPage page = null;
        int action = motionEvent.getActionMasked();
        try {
            switch (action) {
                case MotionEvent.ACTION_DOWN: {
                    PDFViewCtrl viewCtrl = getPdfViewCtrl();
                    if (viewCtrl != null){
                        annot = getCurrentAnnot();
                        if (annot != null && !annot.isEmpty()) {
                            if (AppAnnotUtil.isLocked(annot) || AppAnnotUtil.isReadOnly(annot) || !canAddAnnot()) return false;
                            int type = AppAnnotUtil.getAnnotHandlerType(annot);
                            annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(type);
                            if (annotHandler == null) return false;
                            if (annotHandler.onTouchEvent(pageIndex, motionEvent, annot)) {
                                hideSelectorAnnotMenu(viewCtrl);
                                return true;
                            }
                        }
                        PointF pdfPoint = AppAnnotUtil.getPageViewPoint(viewCtrl, pageIndex, motionEvent);

                        page = viewCtrl.getDoc().getPage(pageIndex);
                        if (page != null) {
                            annot = AppAnnotUtil.createAnnot(page.getAnnotAtDevicePoint(AppUtil.toFxPointF(pdfPoint), ANNOT_SELECT_TOLERANCE,
                                    AppUtil.toMatrix2D(viewCtrl.getDisplayMatrix(pageIndex))));
                        }
                    }
                    break;
                }
                case MotionEvent.ACTION_MOVE:
                case MotionEvent.ACTION_UP:
                case MotionEvent.ACTION_CANCEL:
                    annot = getCurrentAnnot();
                    break;
                default:
                    return false;
            }
            PDFViewCtrl viewCtrl = getPdfViewCtrl();
            if (annot != null && !annot.isEmpty() && viewCtrl != null) {
                if (AppAnnotUtil.isLocked(annot) || AppAnnotUtil.isReadOnly(annot) || !canAddAnnot()) return false;
                int type = AppAnnotUtil.getAnnotHandlerType(annot);
                annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(type);
                if (annotHandler != null && annotHandler.annotCanAnswer(annot)) {
                    hideSelectorAnnotMenu(viewCtrl);
                    return annotHandler.onTouchEvent(pageIndex, motionEvent, annot);
                }
            }
        } catch (PDFException e1) {
            e1.printStackTrace();
        }
        return false;
    }

    public boolean onLongPress(int pageIndex, MotionEvent motionEvent) {
        Annot annot = null;
        AnnotHandler annotHandler = null;
        PDFPage page = null;
        try {
            PDFViewCtrl viewCtrl = getPdfViewCtrl();
            if (viewCtrl != null){
                boolean annotCanceled = false;
                annot = getCurrentAnnot();
                if (annot != null && !annot.isEmpty()) {
                    int type = AppAnnotUtil.getAnnotHandlerType(annot);
                    annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(type);
                    if (annotHandler != null && annotHandler.onLongPress(pageIndex, motionEvent, annot)) {
                        hideSelectorAnnotMenu(viewCtrl);
                        return true;
                    }
                    if (getCurrentAnnot() == null) {
                        annotCanceled = true;
                    }
                }
                PointF pdfPoint = AppAnnotUtil.getPageViewPoint(viewCtrl, pageIndex, motionEvent);
                page = viewCtrl.getDoc().getPage(pageIndex);
                if (page != null && !page.isEmpty()) {
                    annot = AppAnnotUtil.createAnnot(page.getAnnotAtDevicePoint(AppUtil.toFxPointF(pdfPoint), ANNOT_SELECT_TOLERANCE,
                            AppUtil.toMatrix2D(viewCtrl.getDisplayMatrix(pageIndex))));
                }
                if (annot != null && !annot.isEmpty() && AppAnnotUtil.isSupportGroup(annot)) {
                    annot = AppAnnotUtil.createAnnot(((Markup)annot).getGroupHeader());
                }

                if (annot != null && !annot.isEmpty()) {
                    int type = AppAnnotUtil.getAnnotHandlerType(annot);
                    annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(type);
                    if (annotHandler != null && annotHandler.annotCanAnswer(annot)) {
                        if (annotHandler.onLongPress(pageIndex, motionEvent, annot)) {
                            hideSelectorAnnotMenu(viewCtrl);
                            return true;
                        }
                    }
                }

                if (annotCanceled) return true;
            }
            return false;
        } catch (PDFException e1) {
            e1.printStackTrace();
        }

        return false;
    }

    public boolean onSingleTapConfirmed(int pageIndex, MotionEvent motionEvent) {
        Annot annot = null;
        AnnotHandler annotHandler = null;
        PDFPage page = null;
        try {
            PDFViewCtrl viewCtrl = getPdfViewCtrl();
            if (viewCtrl != null){
                boolean annotCanceled = false;
                annot = getCurrentAnnot();
                if (annot != null && !annot.isEmpty()) {
                    int type = AppAnnotUtil.getAnnotHandlerType(annot);
                    annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(type);
                    if (annotHandler != null && annotHandler.onSingleTapConfirmed(pageIndex, motionEvent, annot)) {
                        hideSelectorAnnotMenu(viewCtrl);
                        return true;
                    }
                    if (getCurrentAnnot() == null) {
                        annotCanceled = true;
                    }
                }
                PointF pdfPoint = AppAnnotUtil.getPageViewPoint(viewCtrl, pageIndex, motionEvent);
                page = viewCtrl.getDoc().getPage(pageIndex);
                if (page != null && !page.isEmpty()) {
                    annot = AppAnnotUtil.createAnnot(page.getAnnotAtDevicePoint(AppUtil.toFxPointF(pdfPoint), ANNOT_SELECT_TOLERANCE,
                            AppUtil.toMatrix2D(viewCtrl.getDisplayMatrix(pageIndex))));
                }
                if (annot != null && !annot.isEmpty() && AppAnnotUtil.isSupportGroup(annot)) {
                    annot = AppAnnotUtil.createAnnot(((Markup)annot).getGroupHeader());
                }

                if (annot != null && !annot.isEmpty()) {
                    int type = AppAnnotUtil.getAnnotHandlerType(annot);
                    annotHandler = ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getAnnotHandlerByType(type);
                    if (annotHandler != null && annotHandler.annotCanAnswer(annot)) {
                        if (annotHandler.onSingleTapConfirmed(pageIndex, motionEvent, annot)) {
                            hideSelectorAnnotMenu(viewCtrl);
                            return true;
                        }
                    }
                }

                if (annotCanceled) {
                    return true;
                }
            }
            return false;
        } catch (PDFException e1) {
            e1.printStackTrace();
        }

        return false;
    }

    private void hideSelectorAnnotMenu(PDFViewCtrl pdfViewCtrl) {
        BlankSelectToolHandler blankSelectToolHandler = (BlankSelectToolHandler) ((UIExtensionsManager) pdfViewCtrl.getUIExtensionsManager()).getToolHandlerByType(ToolHandler.TH_TYPE_BLANKSELECT);
        if (blankSelectToolHandler != null) {
            blankSelectToolHandler.dismissMenu();
        }

        TextSelectToolHandler textSelectionTool = (TextSelectToolHandler) ((UIExtensionsManager) pdfViewCtrl.getUIExtensionsManager()).getToolHandlerByType(ToolHandler.TH_TYPE_TEXTSELECT);
        if (textSelectionTool != null) {
            textSelectionTool.mAnnotationMenu.dismiss();
        }
    }

    //for Undo&Redo

    @Override
    public void undo() {
        PDFViewCtrl viewCtrl = getPdfViewCtrl();
        if (viewCtrl != null){
            if (((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getCurrentToolHandler() != null) {
                ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).setCurrentToolHandler(null);
            }

            if (getCurrentAnnot() != null) {
                setCurrentAnnot(null);
            }

            if (haveModifyTasks()) {
                Runnable delayRunnable = new Runnable() {
                    @Override
                    public void run() {
                        undo();
                    }
                };
                viewCtrl.post(delayRunnable);
                return;
            }
        }
        super.undo();
    }

    @Override
    public void redo() {
        PDFViewCtrl viewCtrl = getPdfViewCtrl();
        if (viewCtrl != null){
            if (((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).getCurrentToolHandler() != null) {
                ((UIExtensionsManager) viewCtrl.getUIExtensionsManager()).setCurrentToolHandler(null);
            }

            if (getCurrentAnnot() != null) {
                setCurrentAnnot(null);
            }

            if (haveModifyTasks()) {
                Runnable delayRunnable = new Runnable() {
                    @Override
                    public void run() {
                        redo();
                    }
                };
                viewCtrl.post(delayRunnable);
                return;
            }

        }
        super.redo();
    }

    @Override
    protected String getDiskCacheFolder() {
        PDFViewCtrl viewCtrl = getPdfViewCtrl();
        if (viewCtrl == null){
            return "";
        }
        return viewCtrl.getContext().getCacheDir().getParent();
    }

    @Override
    protected boolean haveModifyTasks() {
        return mHasModifyTask;
    }

    public void setHasModifyTask(boolean hasModifyTask) {
        this.mHasModifyTask = hasModifyTask;
    }

    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (mCurAnnot == null || mCurAnnot.isEmpty()) return false;
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            setCurrentAnnot(null);
            return true;
        }
        return false;
    }

    public boolean onKeyBack() {
        if (mCurAnnot == null || mCurAnnot.isEmpty()) return false;
        setCurrentAnnot(null);
        return true;
    }

    private boolean isOwner(PDFDoc doc) {
        try {
            if (!doc.isEncrypted()) return true;
            if (PDFDoc.e_PwdOwner == doc.getPasswordType()) {
                return true;
            }
        } catch (PDFException e) {
            e.printStackTrace();
        }
        return false;
    }


    private boolean isSign(PDFDoc doc) {
        try {
            int count = doc.getSignatureCount();
            for (int i = 0; i < count; i++) {
                Signature signature = doc.getSignature(i);
                if (signature != null && !signature.isEmpty() && signature.isSigned()) {
                    return true;
                }
            }
        } catch (PDFException e) {

        }
        return false;
    }

    protected void initDocProperties(final PDFDoc doc) {
        if (doc == null || doc.isEmpty()) return;

        try {
            mIsCDRM = doc.isCDRM();
            PDFViewCtrl viewCtrl = getPdfViewCtrl();
            if (viewCtrl != null){
                mUserPermission = viewCtrl.getUserPermission();
                UIExtensionsManager uiExtensionsManager = (UIExtensionsManager)viewCtrl.getUIExtensionsManager();
                if(uiExtensionsManager != null && uiExtensionsManager.getModuleByName(Module.MODULE_NAME_DIGITALSIGNATURE)!= null) {
                    mMDPPermission = getMDPDigitalSignPermissionInDocument(doc);
                }

                mIsSign = isSign(doc);
            }
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    protected void destroy() {
        mAnnotEventListenerList.clear();
        mAnnotEventListenerList = null;
        mActionCallback = null;
        isSetActionCallback = false;
        mPdfViewCtrl.clear();
    }

    public boolean isDocModified() {
        return mIsDocModified;
    }

    public void setDocModified(boolean isModified) {
        mIsDocModified = isModified;
    }

    //for cdrm document
    private int mCDRMPermission = 0xffffffff;
    private boolean mIsAccessPages = false;
    protected ArrayList<Integer> mAccessPages = new ArrayList<>();
    protected String mWrapperContent;
    public void parseACL(String acl) {
        try {
            JSONArray jACLArray = new JSONArray(acl);
            if (jACLArray.length() < 1) return;
            JSONObject data = jACLArray.getJSONObject(0);
            // permission
            if (!data.isNull("permission")) {
                mCDRMPermission = data.getInt("permission");
            }

            //accessPages
            if (!data.isNull("accessPages")) {
                JSONObject json_accessPages = data.getJSONObject("accessPages");
                mIsAccessPages = json_accessPages.getInt("isAccessPages") == 1;

                if (mIsAccessPages) {
                    JSONArray jsonArray_pages = json_accessPages.getJSONArray("Pages");
                    for (int i = 0; i < jsonArray_pages.length(); i ++) {
                        Object o = jsonArray_pages.get(i);
                        if (o instanceof JSONArray) {
                            int start = ((JSONArray) o).getInt(0);
                            int end = ((JSONArray) o).getInt(1);
                            for (int n = start; n <= end; n ++) {
                                mAccessPages.add(n - 1);
                            }
                        } else {
                            mAccessPages.add((Integer) o - 1);
                        }
                    }
                    mWrapperContent = json_accessPages.getString("wrapperContent");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public boolean isAccessPages() {
        return mIsAccessPages;
    }

    public String getWrapperContent() {
        return mWrapperContent;
    }

    public boolean isAccessPage(int pageIndex) {
        if (!mIsAccessPages) return true;
        if (mAccessPages.size() == 0) return false;
        return mAccessPages.contains(pageIndex);
    }

    //For Multiple select annots
    private boolean mIsMultipleSelectAnnots = false;

    public boolean isMultipleSelectAnnots() {
        return mIsMultipleSelectAnnots;
    }

    public void setMultipleSelectAnnots(boolean isMultipleSelectAnnots) {
        mIsMultipleSelectAnnots = isMultipleSelectAnnots;
    }
}
