/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.uiextensions.utils;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Build;
import android.os.LocaleList;

import java.util.Locale;

/**
 * Localization tool class.
 * With this class you can disable localization {@link #enableLocale(Context, boolean)}<br>
 * switch sdk locale {@link #setLocale(Context, Locale)}.<br>
 */
public class LocalizationUtil {

    /**
     * Set the enabled state of localization. If localization is not enabled, the default Locale is English
     *
     * @param context The context to use.  Usually your {@link Application#getApplicationContext()}
     *                or {@link Activity#getApplicationContext()} object.
     * @param enabled True if Localization enabled, false otherwise.
     */
    public static void enableLocale(Context context, boolean enabled) {
        Locale locale = enabled ? getSystemLocale() : Locale.ENGLISH;
        setLocale(context, locale);
    }

    /**
     * You can use this to switch the current locale.
     * Current sdk supports 12 languages,you can swith languages by the following code:<br>
     * 1:english<br>
     * <code>Locale.ENGLISH</code><br>
     * 2:de-CH<br>
     * <code>Locale locale = new Locale("de","CH")</code><br>
     * 3:de-DE<br>
     * <code>Locale.GERMANY</code><br>
     * 4:es-LA<br>
     * <code>Locale locale = new Locale("es","LA")</code><br>
     * 5:fr-FR<br>
     * <code>Locale.FRANCE</code><br>
     * 6:it-IT<br>
     * <code>Locale.ITALY</code><br>
     * 7:ko<br>
     * <code>Locale.KOREAN</code><br>
     * 8:nl-NL<br>
     * <code>Locale locale = new Locale("nl","NL")</code><br>
     * 9:pt-BR<br>
     * <code>Locale locale = new Locale("pt","BR")</code><br>
     * 10:ru-RU<br>
     * <code>Locale locale = new Locale("ru","RU")</code><br>
     * 11:zh-CH<br>
     * <code>Locale.SIMPLIFIED_CHINESE</code> <br>
     * 12:zh-TW<br>
     * <code>Locale.TRADITIONAL_CHINES</code> <br>
     *
     * @param context The context to use.  Usually your {@link Application#getApplicationContext()}
     *                or {@link Activity#getApplicationContext()} object.
     * @param locale  {@link Locale}
     */
    public static void setLocale(Context context, Locale locale) {
        Resources resources = context.getResources();
        Configuration configuration = resources.getConfiguration();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            configuration.setLocale(locale);
            context.createConfigurationContext(configuration);
        } else {
            configuration.locale = locale;
        }
        resources.updateConfiguration(configuration, resources.getDisplayMetrics());
    }

    private static Locale getSystemLocale() {
        Locale locale;
        if (Build.VERSION.SDK_INT < 24) {
            locale = Locale.getDefault();
        } else {
            locale = LocaleList.getDefault().get(0);
        }
        return locale;
    }

}
